@@ -3,7 +3,8 @@
 #' This function fits detection functions to line or point transect data and then (provided that survey information is supplied) calculates abundance and density estimates. The examples below illustrate some basic types of analysis using \code{ds()}.
 #'
 #' @param data a \code{data.frame} containing at least a column called
-#'        \code{distance}. NOTE! If there is a column called \code{size} in
+#'        \code{distance} or a numeric vector containing the distances. 
+#'        NOTE! If there is a column called \code{size} in
 #'        the data then it will be interpreted as group/cluster size, see the
 #'        section "Clusters/groups", below. One can supply data as a "flat file"
 #'        and not supply \code{region.table}, \code{sample.table} and
@@ -13,78 +14,39 @@
 #' @param transect indicates transect type "line" (default) or "point".
 #' @param formula formula for the scale parameter. For a CDS analysis leave this as its default \code{~1}.
 #' @param key key function to use; "hn" gives half-normal (default), "hr" gives hazard-rate and "unif" gives uniform. Note that if uniform key is used, covariates cannot be included in the model.
-#' @param adjustment adjustment terms to use; "cos" gives cosine (default),
-#'        "herm" gives Hermite polynomial and "poly" gives simple polynomial.
-#'        "cos" is recommended. A value of \code{NULL} indicates that no
-#'        adjustments are to be fitted.
-#' @param order orders of the adjustment terms to fit (as a vector/scalar), the
-#'        default value (\code{NULL}) will select via AIC up to order 5. If a single number is given, that number is expanded to be \code{seq(term_min, order, by=1)} where \code{term.min} is the appropriate minimum order for this type of adjustment. For cosine
-#'        adjustments, valid orders are integers greater than 2 (except when a
-#'        uniform key is used, when the minimum order is 1). For Hermite
-#'        polynomials, even integers equal or greater than 2 are allowed and for
-#'        simple polynomials even integers equal or greater than 2 are allowedi (though note these will be multiplied by 2, see Buckland et al, 2001 for details on their specification). By default, AIC selection will try up to 5 adjustments, beyond that you must specify these manually, e.g. \code{order=2:6} and perform your own AIC selection.
-#' @param scale the scale by which the distances in the adjustment terms are
-#'        divided. Defaults to "width", scaling by the truncation
-#'        distance. If the key is uniform only "width" will be used. The other
-#'        option is "scale": the scale parameter of the detection
-#' @param cutpoints if the data are binned, this vector gives the cutpoints of
-#'        the bins. Ensure that the first element is 0 (or the left truncation
-#'        distance) and the last is the distance to the end of the furthest bin.
-#'        (Default \code{NULL}, no binning.)
-#'        Note that if \code{data} has columns \code{distbegin} and
-#'        \code{distend} then these will be used as bins if \code{cutpoints}
-#'        is not specified. If both are specified, \code{cutpoints} has
-#'        precedence.
+#' @param adjustment adjustment terms to use; \code{"cos"} gives cosine (default), \code{"herm"} gives Hermite polynomial and \code{"poly"} gives simple polynomial. \code{"cos"} is recommended. A value of \code{NULL} indicates that no adjustments are to be fitted.
+#' @param order orders of the adjustment terms to fit (as a vector/scalar), the default value (\code{NULL}) will select via AIC up to order 5. If a single number is given, that number is expanded to be \code{seq(term_min, order, by=1)} where \code{term.min} is the appropriate minimum order for this type of adjustment. For cosine adjustments, valid orders are integers greater than 2 (except when a uniform key is used, when the minimum order is 1). For Hermite polynomials, even integers equal or greater than 2 are allowed and for simple polynomials even integers equal or greater than 2 are allowed (though note these will be multiplied by 2, see Buckland et al, 2001 for details on their specification). By default, AIC selection will try up to 5 adjustments, beyond that you must specify these manually, e.g. \code{order=2:6} and perform your own AIC selection.
+#' @param scale the scale by which the distances in the adjustment terms are divided. Defaults to \code{"width"}, scaling by the truncation distance. If the key is uniform only \code{"width"} will be used. The other option is \code{"scale"}: the scale parameter of the detection
+#' @param cutpoints if the data are binned, this vector gives the cutpoints of the bins. Ensure that the first element is 0 (or the left truncation distance) and the last is the distance to the end of the furthest bin. (Default \code{NULL}, no binning.) Note that if \code{data} has columns \code{distbegin} and \code{distend} then these will be used as bins if \code{cutpoints} is not specified. If both are specified, \code{cutpoints} has precedence.
 #' @param monotonicity should the detection function be constrained for monotonicity weakly (\code{"weak"}), strictly (\code{"strict"}) or not at all (\code{"none"} or \code{FALSE}). See Montonicity, below. (Default \code{"strict"}). By default it is on for models without covariates in the detection function, off when covariates are present.
-#' @param dht.group should density abundance estimates consider all groups to be
-#'        size 1 (abundance of groups) \code{dht.group=TRUE} or should the
-#'        abundance of individuals (group size is taken into account),
-#'        \code{dht.group=FALSE}. Default is \code{FALSE} (abundance of
-#'        individuals is calculated).
+#' @param dht.group should density abundance estimates consider all groups to be size 1 (abundance of groups) \code{dht.group=TRUE} or should the abundance of individuals (group size is taken into account), \code{dht.group=FALSE}. Default is \code{FALSE} (abundance of individuals is calculated).
 #' @param region.table \code{data.frame} with two columns:
 #'        \tabular{ll}{ \code{Region.Label} \tab label for the region\cr
 #'                     \code{Area} \tab area of the region\cr}
-#'        \code{region.table} has one row for each stratum. If there is no
-#'        stratification then \code{region.table} has one entry with \code{Area}
-#'        corresponding to the total survey area.
-#' @param sample.table \code{data.frame} mapping the regions to the samples (
-#'        i.e. transects). There are three columns:
+#'        \code{region.table} has one row for each stratum. If there is no stratification then \code{region.table} has one entry with \code{Area} corresponding to the total survey area.
+#' @param sample.table \code{data.frame} mapping the regions to the samples (i.e. transects). There are three columns:
 #'        \tabular{ll}{\code{Sample.Label} \tab label for the sample\cr
 #'                     \code{Region.Label} \tab label for the region that the
 #'                          sample belongs to.\cr
 #'                     \code{Effort} \tab the effort expended in that sample
 #'                          (e.g. transect length).\cr}
-#' @param obs.table \code{data.frame} mapping the individual observations
-#'        (objects) to regions and samples. There should be three columns:
-#'        \tabular{ll}{\code{object} \tab \cr
+#' @param obs.table \code{data.frame} mapping the individual observations (objects) to regions and samples. There should be three columns:
+#'        \tabular{ll}{\code{object} \tab unique numeric identifier for the observation\cr
 #'                     \code{Region.Label} \tab label for the region that the
 #'                          sample belongs to.\cr
 #'                     \code{Sample.Label} \tab label for the sample\cr}
-#' @param convert.units conversion between units for abundance estimation,
-#'        see "Units", below. (Defaults to 1, implying all of the units are
-#'        "correct" already.)
-#'
-#' @param method optimization method to use (any method usable by
-#'        \code{\link{optim}} or \pkg{optimx}). Defaults to
-#'        "nlminb".
-#'
-#' @param debug.level print debugging output. 0=none, 1-3 increasing level of
-#'        debugging output.
-#'
-#' @param quiet surpress non-essential messages (useful for bootstraps etc).
-#'              Default value FALSE.
-#'
-#' @param initial.values a \code{list} of named starting values, see
-#'        \code{\link{mrds-opt}}. Only allowed when AIC term selection is not used.
-#'
+#' @param convert.units conversion between units for abundance estimation, see "Units", below. (Defaults to 1, implying all of the units are "correct" already.)
+#' @param method optimization method to use (any method usable by \code{\link{optim}} or \pkg{optimx}). Defaults to \code{"nlminb"}.
+#' @param debug.level print debugging output. \code{0}=none, \code{1-3} increasing levels of debugging output.
+#' @param quiet surpress non-essential messages (useful for bootstraps etc). Default value \code{FALSE}.
+#' @param initial.values a \code{list} of named starting values, see \code{\link{mrds-opt}}. Only allowed when AIC term selection is not used.
 #' @return a list with elements:
 #'        \tabular{ll}{\code{ddf} \tab a detection function model object.\cr
 #'                     \code{dht} \tab abundance/density information (if survey
 #'                      region data was supplied, else \code{NULL}).}
 #'
 #' @section Details:
-#'
-#' If abundance estimates are required then the \code{data.frame}s \code{region.table} and \code{sample.table} must be supplied. If \code{data} does not contain the columns \code{Region.Label} and \code{Sample.Label} thenthe \code{data.frame} \code{obs.table} must also be supplied. Note that stratification only applies to abundance estimates and not at the detection function level.
+#' If abundance estimates are required then the \code{data.frame}s \code{region.table} and \code{sample.table} must be supplied. If \code{data} does not contain the columns \code{Region.Label} and \code{Sample.Label} then the \code{data.frame} \code{obs.table} must also be supplied. Note that stratification only applies to abundance estimates and not at the detection function level.
 #'
 #' @section Clusters/groups:
 #'  Note that if the data contains a column named \code{size} and \code{region.table}, \code{sample.table} and \code{obs.table} are supplied, cluster size will be estimated and density/abundance will be based on a clustered analsis of the data. Setting this column to be \code{NULL} will perform a non-clustred analysis (for example if "\code{size}" means something else in your dataset).
@@ -184,6 +146,11 @@
 #' # detection function
 #' ds.model.hr.trunc <- ds(tee.data,truncation="10%",key="hr",adjustment=NULL)
 #' summary(ds.model.hr.trunc)
+#'
+#' # compare AICs between these models:
+#' AIC(ds.model)
+#' AIC(ds.model.cos2)
+#' AIC(ds.model.cos23)
 #'}
 #'
 ds <- function(data, truncation=ifelse(is.null(cutpoints),
@@ -206,7 +173,13 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
   # this routine just creates a call to mrds, it's not very exciting
   # or fancy, it does do a lot of error checking though
 
-
+  # check the data, format into the correct tables if we have a flat file
+  data <- checkdata(data, region.table, sample.table, obs.table, formula)
+  region.table <- data$region.table
+  sample.table <- data$sample.table
+  obs.table    <- data$obs.table
+  data         <- data$data
+  
   # truncation
   if(is.null(truncation)){
     stop("Please supply truncation distance or percentage.")
@@ -229,7 +202,8 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
         }else if(is.character(truncation$left) & length(truncation$left)==1){
           # % string to number
           truncation$left <- as.numeric(sub("%","",truncation$left))
-          left <- quantile(data$distance,probs=(truncation$left/100))
+          left <- quantile(data$distance, probs=truncation$left/100,
+                           na.rm=TRUE)
         }else{
           stop("Truncation must be supplied as a single number/string or a list with elements \"left\" and \"right\".")
         }
@@ -238,8 +212,9 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
           width <- truncation$right
         }else if(is.character(truncation$right) & length(truncation$right)==1){
           # % string to number
-          truncation$right <- as.numeric(sub("%","",truncation$right))
-          width <- quantile(data$distance,probs=1-(truncation$right/100))
+          truncation$right <- as.numeric(sub("%", "", truncation$right))
+          width <- quantile(data$distance, probs=1-(truncation$right/100),
+                            na.rm=TRUE)
         }else{
           stop("Truncation must be supplied as a single number/string or a list with elements \"left\" and \"right\".")
         }
@@ -248,26 +223,19 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
       }
 
     # just right truncation
-    }else if(is.double(truncation) & length(truncation)==1){
+    }else if(is.numeric(truncation) & length(truncation)==1){
       width <- truncation
       left <- NULL
     }else if(is.character(truncation) & length(truncation)==1){
       # % string to number
       truncation <- as.numeric(sub("%","",truncation))
-      width <- quantile(data$distance,probs=1-(truncation/100))
+      width <- quantile(data$distance, probs=1-(truncation/100), na.rm=TRUE)
       left <- NULL
     }else{
       stop("Truncation must be supplied as a single number/string or a list with elements \"left\" and \"right\".")
     }
   }
 
-  # check the data, format into the correct tables if we have a flat file
-  data <- checkdata(data, region.table, sample.table, obs.table, formula)
-  region.table <- data$region.table
-  sample.table <- data$sample.table
-  obs.table    <- data$obs.table
-  data         <- data$data
-
   ### binning
   if(is.null(cutpoints)){
     if(any(names(data)=="distend") & any(names(data)=="distbegin")){
@@ -391,7 +359,7 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
       # if there are covariates then don't do the AIC search
       if(formula != ~1){
         aic.search <- FALSE
-        message("Cannot perform AIC adjustment term selection when covariates are used.")
+        message("Model contains covariate term(s): no adjustment terms will be included.")
       }else{
       # otherwise go ahead and set up the candidate adjustment orders
         aic.search <- TRUE
@@ -561,7 +529,11 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
           # return the last model and stop looking.
           if(model$criterion>last.model$criterion){
             model <- last.model
-            message(paste0("\n\n",model$name.message," selected!"))
+            # capitalise!
+            model_name <- model$name.message
+            model_name <- paste0(toupper(substring(model_name, 1, 1)),
+                                substring(model_name, 2))
+            message(paste0("\n", model_name, " selected."))
             break
           }else{
             # otherwise keep this, best model
@@ -596,22 +568,30 @@ ds <- function(data, truncation=ifelse(is.null(cutpoints),
     # if obs.table is not supplied, then data must have the Region.Label and
     # Sample.Label columns
     if(is.null(obs.table)){
-      if(c("Region.Label","Sample.Label") %in% names(data)){
-        message("No obs.table supplied but data does not have Region.Label or Sample.Label columns, only estimating detection function.\n")
+      if(all(c("Region.Label","Sample.Label") %in% names(data))){
+        dht.res <- dht(model,region.table,sample.table,
+                     options=list(#varflag=0,group=TRUE,
+                                  group=dht.group,
+                                  convert.units=convert.units),se=TRUE)
+
+      }else{
+        message("No obs.table supplied nor does data have Region.Label and Sample.Label columns, only estimating detection function.\n")
+        dht.res <- NULL
       }
+    }else{
+
+      # from ?dht:
+      # For animals observed in tight clusters, that estimator gives the
+      # abundance of groups (group=TRUE in options) and the abundance of
+      # individuals is estimated as s_1/p_1 + s_2/p_2 + ... + s_n/p_n, where
+      # s_i is the size (e.g., number of animals in the group) of each
+      # observation(group=FALSE in options).
+
+      dht.res <- dht(model,region.table,sample.table,obs.table,
+                   options=list(#varflag=0,group=TRUE,
+                                group=dht.group,
+                                convert.units=convert.units),se=TRUE)
     }
-
-    # from ?dht:
-    # For animals observed in tight clusters, that estimator gives the
-    # abundance of groups (group=TRUE in options) and the abundance of
-    # individuals is estimated as s_1/p_1 + s_2/p_2 + ... + s_n/p_n, where
-    # s_i is the size (e.g., number of animals in the group) of each
-    # observation(group=FALSE in options).
-
-    dht.res <- dht(model,region.table,sample.table,obs.table,
-                 options=list(#varflag=0,group=TRUE,
-                              group=dht.group,
-                              convert.units=convert.units),se=TRUE)
   }else{
     # if no information on the survey area was supplied just return
     # the detection function stuff
